(******************************************************************************************************)
(*												      *)
(*	Dr. William T. Verts (C) April 28, 1996							      *)
(*												      *)
(*	Program to demonstrate simple frame-based animation on VGA 320x200 256-color mode	      *)
(*												      *)
(******************************************************************************************************)
{$R-}
{$I-}
{$N-}
{$E-}

Program Animation ;

Uses	Crt,
	Dos,
	VGA_320 ;

Const	Bell		= ^G					;
	Radius		= 40					;

Type	VGA_Pointer	= ^VGA_Array				;

Var	Table		: Array [1..16] Of VGA_Pointer		;
	Top		: 0..16					;
	Saved_Palette	: Color_Table				;
	P		: Color_Table				;

	Wait		: Integer				;
	I		: Integer				;
	X, Y, Step	: Integer				;
	CH		: Char					;
	Done_Flag	: Boolean				;

Begin	(*!*)
	(*--------------------------------------------------------------------*)

	TextMode	(C80) ;
	TextColor	(White) ;
	TextBackground	(Blue) ;
	ClrScr ;
	Writeln		;
	Writeln		('                  Simple VGA 320x200 256-Color Animation Demo') ;
	Writeln		('               Copyright Dr. William T. Verts (C) April 28, 1996') ;
	Writeln		;
	Writeln		('  This program creates as many off-screen  frames  as  memory  allows,  plots') ;
	Writeln		('  images  for  each,  then  animates  the images by successively copying them') ;
	Writeln		('  into the VGA video  frame  buffer  as  fast  as  possible.   Delay  between') ;
	Writeln		('  frames  may  be controlled by hitting the digit keys [0..9] on the keyboard') ;
	Writeln		('  (0 delay is fastest, 9 delay is slowest).  Hit Escape or Q to quit program.') ;
	Write		(Bell) ;
	Readln		;

	(*--------------------------------------------------------------------*)
	(* Allocate as many 64000 byte VGA frames as will fit into memory.    *)

	Top	:= Zero ;
	FillChar (Table, SizeOf(Table), Zero) ;

	While (Top < 16) And (MaxAvail > SizeOf(VGA_Array) + 10240) Do
		Begin	(***)
			Inc	(Top) ;
			New	(Table[Top]) ;
			FillChar(Table[Top]^, SizeOf(VGA_Array), Zero) ;
		End ;

	If Top < 2 Then
		Begin	(***)
			Writeln	('Not enough memory to run program') ;
			Write	(Bell) ;
			Readln	;
			Halt ;
		End ;

	(*--------------------------------------------------------------------*)
	(* Start graphics mode, save the existing palette, then clear it so   *)
	(* all plotting actions will be hidden (all colors set to black).     *)

	InitGraph ;
	Get_Palette (Saved_Palette) ;
	FillChar (P, SizeOf(P), Zero) ;
	Put_Palette (P) ;

	(*--------------------------------------------------------------------*)
	(* For each memory frame, plot a background of a couple of rectangles,*)
	(* then plot a circle over top in a location dependent on the frame   *)
	(* number.  Since the palette has been cleared, nothing is visible.   *)

	Step	:= (GetMaxX - (2 * Radius)) Div Pred(Top) ;

	For I := 1 To Top Do
		Begin	(***)
			X	:= Radius + Pred(I) * Step ;
			Y	:= GetMaxY Div 2 ;

			Fill_Screen	(1) ;

			Filled_Rectangle(20, 20, GetMaxX - 100, GetMaxY - 20, 2) ;
			Rectangle	(20, 20, GetMaxX - 100, GetMaxY - 20, 3) ;
			Line		(20, 20, GetMaxX - 100, GetMaxY - 20, 3) ;
			Line		(GetMaxX - 100, 20, 20, GetMaxY - 20, 3) ;

			Filled_Rectangle(50, 40, GetMaxX -  50, GetMaxY - 40, 4) ;
			Rectangle	(50, 40, GetMaxX -  50, GetMaxY - 40, 5) ;
			Line		(50, 40, GetMaxX -  50, GetMaxY - 40, 5) ;
			Line		(GetMaxX -  50, 40, 50, GetMaxY - 40, 5) ;

			Filled_Circle	(X, Y, Radius, 6) ;
			Circle		(X, Y, Radius, 7) ;

			Table[I]^	:= Screen_Array ;
		End ;

	(*--------------------------------------------------------------------*)
	(* Define and load the color palette.				      *)

	ClearDevice ;
	With P[1] Do Begin Red :=   0 ; Green :=   0 ; Blue := 255 End ;	(* Background  *)
	With P[2] Do Begin Red :=   0 ; Green := 255 ; Blue :=   0 End ;	(* Box #1 Body *)
	With P[3] Do Begin Red := 255 ; Green :=   0 ; Blue := 255 End ;	(* Box #1 Rim  *)
	With P[4] Do Begin Red :=   0 ; Green := 255 ; Blue := 255 End ;	(* Box #2 Body *)
	With P[5] Do Begin Red := 255 ; Green := 255 ; Blue := 255 End ;	(* Box #2 Rim  *)
	With P[6] Do Begin Red := 255 ; Green :=   0 ; Blue :=   0 End ;	(* Circle Body *)
	With P[7] Do Begin Red := 255 ; Green := 255 ; Blue :=   0 End ;	(* Circle Rim  *)
	Put_Palette (P) ;

	(*--------------------------------------------------------------------*)
	(* Animate the images, slamming them successively into the VGA frame  *)
	(* buffer, then waiting a user-specified time.			      *)

	I		:= 1 ;
	Done_Flag	:= False ;
	Wait		:= 0 ;

	Repeat	(***)
		While KeyPressed Do
			Begin	(***)
				CH	:= UpCase(ReadKey) ;
				Case CH Of
			#0	 :	ReadKey ; (* Ignore function keys *)
			#27, 'Q' :	Done_Flag := True ;
			'0'..'9' :	Wait := (Ord(CH) - Ord('0')) * 75 ;
			Else		Begin End ;
				End ;	(* CASE *)
			End ;

		Screen_Array := Table[I]^ ;
		If I < Top Then Inc (I) Else I := 1 ;
		Delay (Wait) ;
	Until Done_Flag ;

	(*--------------------------------------------------------------------*)
	(* Restore the existing palette, then shut down graphics mode.	      *)

	Put_Palette (Saved_Palette) ;
	CloseGraph ;
End.
